/* ===================================================================
   \[Xt@C
   =================================================================== */

/* ********************************************************** */
/* Include */
/* ********************************************************** */
#include "AdcSampleApl.h"
#include "system.h"
#include <math.h>

/* ********************************************************** */
/* Structure & Define */
/* ********************************************************** */
/* ^C}[ */
typedef enum {
  ADC_TIMER_STOP,
  ADC_TIMER_DOING
} ADC_TIMER_STATE;

/* DAC CH` */
#define DAC_APL_CH_A    (0x01)
#define DAC_APL_CH_B    (0x05)

/* ADC CH ` */
#define ADC_APL_CH_A    (0x00)
#define ADC_APL_CH_B    (0x03)

/* bݒl */
#define ADC_TIME            (5) /* ݒύX(5ms) */
#define ADC_INTVAL_TIME_MS  (10) /* Interval Timer msl */

/* ********************************************************** */
/* Local Variable */
/* ********************************************************** */
static ADC_TIMER_STATE adc_timer_state;
static int adc_timer_cnt;

/* ********************************************************** */
/* Local ProtoType Define */
/* ********************************************************** */
static int adc_apl_get_sinwave(int x);
static void adc_apl_set_timer (void);
static void adc_apl_output_data (int ch_a_data, int ch_b_data);

/* ================================================================== */
/* ********************************************************** */
/* Src (Internal API Function) */
/* ********************************************************** */
/* ********************************************************** */
/* Tv@F@ADCAv̏                                */
/* ֐@F@AdcApl_Init                                      */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
AdcApl_Init (void)
{
  adc_timer_state = ADC_TIMER_STOP;
  adc_timer_cnt   = 0;

	IOWR(PINMUX_BASE, 0, 0xddff);	// SPI1, SPI0, UART1, UART0
	IOWR(PINMUX_BASE, 1, 0x0305);	// PWM1, PWM2, SPI2
  return;
}


/* ********************************************************** */
/* Tv@F@ADCAv̎s                                  */
/* ֐@F@AdcApl_Exec                                      */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
AdcApl_Exec (void)
{
  if (adc_timer_state == ADC_TIMER_STOP)
	{
	  static int ch_a_cnt = 0;
	  static int ch_b_cnt = 90;
	  int ch_a_data, ch_b_data;
	  int adc_ch_a_data = 0;
	  int adc_ch_b_data = 0;
	  unsigned char addr;

	  /* mFp̃^C}[ݒ */
	  adc_apl_set_timer ();
	  /* ^C}[쒆ݒ */
	  adc_timer_state = ADC_TIMER_DOING;


	  /* CH : A */
	  /* Sing擾 */
	  ch_a_data = adc_apl_get_sinwave (ch_a_cnt);
	  addr = DAC_APL_CH_A;
	  /* DAC : SPI o */
	  {
		unsigned char buf[2];
		buf[0] = (addr << 4) | (ch_a_data >> 4);
		buf[1] = (ch_a_data << 4);

		spi_xfer ("DAC082S085", &buf[0], WR, sizeof(buf));
	  }
	  ch_a_cnt++;
	  if (ch_a_cnt >= 360)
		{
		  ch_a_cnt = 0;
		}

	  /* CH : B */
	  /* Sing擾 */
	  ch_b_data = adc_apl_get_sinwave (ch_b_cnt);
	  addr = DAC_APL_CH_B;
	  /* DAC : SPI o */
	  {
		unsigned char buf[2];
		buf[0] = (addr << 4) | (ch_b_data >> 4);
		buf[1] = (ch_b_data << 4);

		spi_xfer ("DAC082S085", &buf[0], WR, sizeof(buf));
	  }

	  ch_b_cnt++;
	  if (ch_b_cnt >= 360)
		{
		  ch_b_cnt = 0;
		}

	  /* CH : A ADCl擾 */
	  addr = ADC_APL_CH_A;
	  {
		unsigned char buf[2];
		buf[0] = (unsigned char)((addr << 3) & 0x38);
		buf[1] = 0;
		/* CH w */
		spi_xfer ("ADC108S022", &buf[0], WR, 2);
	  }

	  spi_xfer ("ADC108S022", (unsigned char *)&adc_ch_a_data, RE, 2);
	  {
		unsigned int tmp1, tmp2;

		tmp1 = (adc_ch_a_data & 0xff00) >> 8;
		tmp2 = (adc_ch_a_data & 0x00ff) << 8;
		adc_ch_a_data = (tmp2 | tmp1); 
	  }


	  /* CH : B ADCl擾 */
	  addr = ADC_APL_CH_B;
	  {
		unsigned char buf[2];
		buf[0] = (unsigned char)((addr << 3) & 0x38);
		buf[1] = 0;
		/* CH w */
		spi_xfer ("ADC108S022", &buf[0], WR, 2);
	  }

	  spi_xfer ("ADC108S022", (unsigned char *)&adc_ch_b_data, RE, 2);
	  {
		unsigned int tmp1, tmp2;

		tmp1 = (adc_ch_b_data & 0xff00) >> 8;
		tmp2 = (adc_ch_b_data & 0x00ff) << 8;
		adc_ch_b_data = (tmp2 | tmp1); 
	  }

	  /* UART ֏o */
	  adc_apl_output_data (adc_ch_a_data, adc_ch_b_data);
	}
  return;
}

/* ********************************************************** */
/* Tv@F@ADCAv Timer荞                          */
/* ֐@F@AdcApl_Irq_Timer                                 */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
AdcApl_Irq_Timer (void)
{
  if (adc_timer_state == ADC_TIMER_DOING)
	{
	  if (adc_timer_cnt <= 0)
		{
		  adc_timer_state = ADC_TIMER_STOP;
		}
	  else
		{
		  adc_timer_cnt--;
		}
	}

  return;
}

/* ********************************************************** */
/* Tv@F@ADCAv Timerݒ                              */
/* ֐@F@AdcApl_Set_Timer                                 */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
AdcApl_Set_Timer (void)
{
  adc_apl_set_timer ();
  /* ^C}[쒆ݒ */
  adc_timer_state = ADC_TIMER_DOING;

  return;
}

/* ********************************************************** */
/* Tv@F@ADCAv Timer~ݒ                          */
/* ֐@F@AdcApl_Stop_Timer                                */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
AdcApl_Stop_Timer (void)
{
  adc_timer_cnt = 0;
  adc_timer_state = ADC_TIMER_STOP;
  return;
}


/* ================================================================== */
/* ********************************************************** */
/* Src (Local Function) */
/* ********************************************************** */
/* ********************************************************** */
/* Tv@F@ADCAv Singf[^̎擾                      */
/* ֐@F@adc_apl_get_sinwave                              */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static int 
adc_apl_get_sinwave(int x)
{
  double y;
  y = 127 * sin((double) x * M_PI / 180.0) + 128.0;
  return (int) y;
}

/* ********************************************************** */
/* Tv@F@ADCAṽ^C}[lݒ                        */
/* ֐@F@adc_apl_set_timer                                */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
adc_apl_set_timer (void)
{
  adc_timer_cnt = ADC_TIME * ADC_INTVAL_TIME_MS;  /* 5ms : l͎b */

  return;
}

/* ********************************************************** */
/* Tv@F@ADCAv ADClUARTo                        */
/* ֐@F@adc_apl_output_data                              */
/* @F  int ch_a_data                                    */
/*           int ch_b_data                                    */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
adc_apl_output_data (int ch_a_data, int ch_b_data)
{
  char buf[10];
  memset (&buf[0], 0, sizeof(buf));
  sprintf (&buf[0], "%2x", ch_a_data >> 4);
  sprintf (&buf[2], "%2x", ch_b_data >> 4);

  buf[4] = '\r';
  buf[5] = '\n';

  uart_xfer ("UART0", &buf[0], WR, 6);
  return;
}
