/* ===================================================================
   \[Xt@C
   =================================================================== */

/* ********************************************************** */
/* Include */
/* ********************************************************** */
#include "BuzzerSampleApl.h"

/* ********************************************************** */
/* Structure & Define */
/* ********************************************************** */
/* uU[ */
typedef enum {
  BUZZER_PWM_STOP,
  BUZZER_PWM_DOING
} BUZZER_APL_STATE;

/* ^C}[ */
typedef enum {
  BUZZER_TIMER_STOP,
  BUZZER_TIMER_DOING
} BUZZER_TIMER_STATE;


/* bݒl */
#define BUZZER_PWM_DUTY   (50)   /* Dutyl */
#define BUZZER_TIME       (250)  /* 炷(250ms) */
#define BUZZER_INTVAL_TIME_MS  (10) /* Interval Timer msl */


/* ********************************************************** */
/* Local Variable */
/* ********************************************************** */
static BUZZER_APL_STATE buzzer_state;
static BUZZER_TIMER_STATE buzzer_timer_state;
static int buzzer_timer_cnt;
static int buzzer_set_on;

/* ********************************************************** */
/* Local ProtoType Define */
/* ********************************************************** */
static void buzzer_apl_set_timer (void);
static int buzzer_apl_is_input (void);

/* ================================================================== */
/* ********************************************************** */
/* Src (Internal API Function) */
/* ********************************************************** */

/* ********************************************************** */
/* Tv@F@uU[Av̏                             */
/* ֐@F@BuzzerApl_Init                                   */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
BuzzerApl_Init (void)
{
  buzzer_state       = BUZZER_PWM_STOP;
  buzzer_timer_state = BUZZER_TIMER_STOP;
  buzzer_timer_cnt   = 0;
  buzzer_set_on      = false;

  pwm_duty (PWM_0_NAME, BUZZER_PWM_DUTY);
  return;
}


/* ********************************************************** */
/* Tv@F@uU[Av̎s                               */
/* ֐@F@BuzzerApl_Exec                                   */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
BuzzerApl_Exec (void)
{
  /* Ԃ̊mF */
  if (buzzer_state == BUZZER_PWM_DOING)
	{
	  /* ^C}[~mF */
	  if (buzzer_timer_state == BUZZER_TIMER_STOP)
		{
		  pwm_stop(PWM_0_NAME);
		  buzzer_state = BUZZER_PWM_STOP;
		}
	}
  else
	{
	  /* SW͊mF */
	  if (buzzer_apl_is_input () == true)
		{
		  pwm_start (PWM_0_NAME);

		  /* ^C}[X^[g */
		  buzzer_apl_set_timer ();

		  /* ^C}[쒆ݒ */
		  buzzer_timer_state = BUZZER_TIMER_DOING;

		  buzzer_state = BUZZER_PWM_DOING;
		}
	}

  return;
}


/* ********************************************************** */
/* Tv@F@uU[Av Timer荞                       */
/* ֐@F@BuzzerApl_Irq_Timer                              */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
BuzzerApl_Irq_Timer (void)
{
  if (buzzer_timer_state == BUZZER_TIMER_DOING)
	{
	  if (buzzer_timer_cnt <= 0)
		{
		  buzzer_timer_state = BUZZER_TIMER_STOP;
		}
	  else
		{
		  buzzer_timer_cnt--;
		}
	}

  return;
}

/* ********************************************************** */
/* Tv@F@uU[Av uU[ONv                        */
/* ֐@F@BuzzerApl_Set_BuzzerOn                           */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
BuzzerApl_Set_BuzzerOn (void)
{
  buzzer_set_on = true;
  return;
}

/* ================================================================== */
/* ********************************************************** */
/* Src (Local Function) */
/* ********************************************************** */

/* ********************************************************** */
/* Tv@F@uU[Aṽ^C}[lݒ                     */
/* ֐@F@buzzer_apl_set_timer                             */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
buzzer_apl_set_timer (void)
{
  buzzer_timer_cnt = BUZZER_TIME * BUZZER_INTVAL_TIME_MS;  /* 250ms :l͎b */
  return;
}

/* ********************************************************** */
/* Tv@F@uU[炷Input邩ǂ                */
/* ֐@F@buzzer_apl_is_input                              */
/* @F@Ȃ                                             */
/* ߂lF@true :  / false F Ȃ                      */
/* ********************************************************** */
static int 
buzzer_apl_is_input (void)
{
  if (buzzer_set_on == true)
	{
	  buzzer_set_on = false;
	  return true;
	}


  if (pio_input (PD4) == 1)
	{
	  return true;
	}

  if (pio_input (PD5) == 1)
	{
	  return true;
	}

  if (pio_input (PD6) == 1)
	{
	  return true;
	}

  if (pio_input (PD7) == 1)
	{
	  return true;
	}

  return false;
}
