/* ===================================================================
   \[Xt@C
   =================================================================== */

/* ********************************************************** */
/* Include */
/* ********************************************************** */
#include "EtherSampleApl.h"
#include "ioLibrary/Ethernet/socket.h"
#include "limits.h"
#include <stdlib.h>

#include "BuzzerDemoApl.h"
#include "ServoDemoApl.h"
#include "SteppingDemoApl.h"
#include "LedDemoApl.h"
#include "AdcDemoApl.h"
#include "RtcDemoApl.h"

/* ********************************************************** */
/* Structure & Define */
/* ********************************************************** */
#define SOCK_TCPS        0
#define SOCK_UDPS        1


/* ********************************************************** */
/* Local Variable */
/* ********************************************************** */
static wiz_NetInfo ether_net_info = { .mac = {0x00, 0x08, 0xdc, 0xab, 0xcd, 0xef},
									  .ip = {192, 168, 0, 231},
									  .sn = {255, 255, 255, 0},
									  .gw = {192, 168, 1, 1},
									  .dns = {0, 0, 0, 0},
									  .dhcp = NETINFO_STATIC };

#define ETHER_MAX_BUF  2048
static unsigned char ether_buf[ETHER_MAX_BUF];
static char print_buf[100]; 


typedef struct _ether_cmd_tbl_ {
  char *cmd;
  void (*func)(unsigned char data0, unsigned char data1, int type, unsigned char *addr, unsigned short port);
} ETHER_CMD_TBL;

/* ********************************************************** */
/* Local ProtoType Define */
/* ********************************************************** */
static int ether_apl_recv_tcp (unsigned short port);
static int ether_apl_recv_udp (unsigned short port);
static void ether_apl_analyze_recv_data (unsigned char *buf, int type, unsigned char *addr, unsigned short port);
static void ether_cmd_initialize (unsigned char data0, unsigned char data1, int type, unsigned char *addr, unsigned short port);
static void ether_cmd_rtc_write (unsigned char data0, unsigned char data1, int type, unsigned char *addr, unsigned short port);
static void ether_cmd_rtc_read (unsigned char data0, unsigned char data1, int type, unsigned char *addr, unsigned short port);
static void ether_cmd_lcd_light (unsigned char data0, unsigned char data1, int type, unsigned char *addr, unsigned short port);
static void ether_cmd_led_write (unsigned char data0, unsigned char data1, int type, unsigned char *addr, unsigned short port);
static void ether_cmd_led_read (unsigned char data0, unsigned char data1, int type, unsigned char *addr, unsigned short port);
static void ether_cmd_servo_ctl (unsigned char data0, unsigned char data1, int type, unsigned char *addr, unsigned short port);
static void ether_cmd_step_ctl (unsigned char data0, unsigned char data1, int type, unsigned char *addr, unsigned short port);
static void ether_cmd_auto_mode (unsigned char data0, unsigned char data1, int type, unsigned char *addr, unsigned short port);



static const ETHER_CMD_TBL ether_cmd_tbl[] = {
  {"IT", ether_cmd_initialize	},
  {"RW", ether_cmd_rtc_write	},
  {"RR", ether_cmd_rtc_read  	},
  {"BC", ether_cmd_lcd_light 	},
  {"LW", ether_cmd_led_write 	},
  {"LR", ether_cmd_led_read 	},
  {"SV", ether_cmd_servo_ctl	},
  {"SP", ether_cmd_step_ctl 	},
  {"AU", ether_cmd_auto_mode 	}
};


/* ================================================================== */
/* ********************************************************** */
/* Src (Internal API Function) */
/* ********************************************************** */
/* ********************************************************** */
/* Tv@F@EtherAv̏                              */
/* ֐@F@EtherApl_Init                                    */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
EtherApl_Init (void)
{
  memset (&ether_buf[0], 0, sizeof(ether_buf));
  memset (&print_buf[0], 0, sizeof(print_buf));

  W5500_Init ();


  return;
}

/* ********************************************************** */
/* Tv@F@EtherAv@lbgڑݒ                      */
/* ֐@F@EtherApl_Set_NetConfig                           */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
EtherApl_Set_NetConfig (void)
{
  /* wizchip netconf */
  ctlnetwork(CN_SET_NETINFO, (void*) &ether_net_info);

  return;
}

/* ********************************************************** */
/* Tv@F@EtherAv̎s                                */
/* ֐@F@EtherApl_Exec                                    */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
EtherApl_Exec (void)
{
  int ret;

  /* TCP M */
  ret = ether_apl_recv_tcp (5000);
  if (ret < 0)
	{
	  sprintf (&print_buf[0], "SOCKET ERROR(TCP) : %d\r\n", ret);
	  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
	}

  /* UDP M */
  ret = ether_apl_recv_udp (3000);
  if (ret < 0)
	{
	  sprintf (&print_buf[0], "SOCKET ERROR(UDP) : %d\r\n", ret);
	  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
	}

  return;
}



/* ================================================================== */
/* ********************************************************** */
/* Src (Local Function) */
/* ********************************************************** */
/* ********************************************************** */
/* Tv@F@TCPM		                              */
/* ֐@F@ether_apl_recv_tcp                               */
/* @F@unsigned short port : |[gԍ                 */
/* ߂lF@int                                              */
/* ********************************************************** */
static int 
ether_apl_recv_tcp (unsigned short port)
{
  int ret;
  unsigned short size = 0;

  switch (getSn_SR(SOCK_TCPS))
	{
	case SOCK_ESTABLISHED:
	  if (getSn_IR(SOCK_TCPS) & Sn_IR_CON)
		{
		  sprintf (&print_buf[0], "TCP Connected\r\n");
		  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
		  setSn_IR (SOCK_TCPS, Sn_IR_CON);
		}
	  
	  /* f[^MTCY̊mF */
	  if ((size = getSn_RX_RSR(SOCK_TCPS)) > 0)
		{
		  if (size > ETHER_MAX_BUF)
			{
			  size = ETHER_MAX_BUF;
			}
		  /* f[^M */
		  ret = recv (SOCK_TCPS, &ether_buf[0], size);
		  if (ret <= 0)
			{
			  return ret;
			}
		  /* Mf[^̉͂Ǝs */
		  ether_apl_analyze_recv_data (&ether_buf[0], SOCK_TCPS, NULL, 0);
		}
	  break;
	case SOCK_CLOSE_WAIT:
	  sprintf (&print_buf[0], "TCP :CloseWait\r\n");
	  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
	  if ((ret = disconnect(SOCK_TCPS)) != SOCK_OK)
		{
		  return ret;
		}
	  sprintf (&print_buf[0], "TCP :Closed\r\n");
	  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
	  break;
	case SOCK_INIT:
	  sprintf (&print_buf[0], "TCP :Listen, port [%d]\r\n", port);
	  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
	  if ((ret = listen(SOCK_TCPS)) != SOCK_OK)
		{
		  return ret;
		}
	  break;
	case SOCK_CLOSED:
	  sprintf (&print_buf[0], "TCP :LBTStart\r\n");
	  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
	  if ((ret = socket(SOCK_TCPS, Sn_MR_TCP, port, 0x00)) != SOCK_TCPS)
		{
		  return ret;
		}
	  sprintf (&print_buf[0], "TCP :Opened\r\n");
	  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
	  break;
	default:
	  break;
	}

  return 1;
}


/* ********************************************************** */
/* Tv@F@UDPM		                              */
/* ֐@F@ether_apl_recv_udp                               */
/* @F@unsigned short port : |[gԍ                 */
/* ߂lF@int                                              */
/* ********************************************************** */
static int 
ether_apl_recv_udp (unsigned short port)
{
  int ret;
  unsigned short size;
  unsigned char destip[4];
  unsigned short destport;

  switch (getSn_SR(SOCK_UDPS))
	{
	case SOCK_UDP:
	  if ((size = getSn_RX_RSR(SOCK_UDPS)) > 0)
		{
		  if (size > ETHER_MAX_BUF)
			{
			  size = ETHER_MAX_BUF;
			}
		  ret = recvfrom (SOCK_UDPS, &ether_buf[0], size, &destip[0], &destport);
		  if (ret < 0)
			{
			  sprintf (&print_buf[0], "UDP : recvfrom error. %d\r\n", ret);
			  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
			  return ret;
			}
		  /* Mf[^̉͂Ǝs */
		  ether_apl_analyze_recv_data (&ether_buf[0], SOCK_UDPS, &destip[0], destport);
		}
	  break;
	case SOCK_CLOSED:
	  sprintf (&print_buf[0], "UDP :LBUStart\r\n");
	  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
	  if ((ret = socket(SOCK_UDPS, Sn_MR_UDP, port, 0x00)) != SOCK_UDPS)
		{
		  return ret;
		}

	  sprintf (&print_buf[0], "UDP :Opened, port [%d]\r\n", port);
	  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
	  break;
	default:
	  break;
	}

  return 1;
}



/* ********************************************************** */
/* Tv@F@CMD͏		                              */
/* ֐@F@ether_apl_analyze_recv_data                      */
/* @F@unsigned char *buf : CMDi[                   */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
ether_apl_analyze_recv_data (unsigned char *buf, int type, unsigned char *addr, unsigned short port)
{
  int i;

  /* Buffer Ƀf[^Ƃ͎s */
  /* 擪uFvł邱Ƃ̊mF */
  if (buf[0] != ':')
	{
	  return;
	}
  /* CMD  */
  for (i = 0; i < sizeof(ether_cmd_tbl)/sizeof(ether_cmd_tbl[0]); i++)
	{
	  if (strncmp ((char *)&(buf[1]), ether_cmd_tbl[i].cmd, 2) == 0)
		{
		  int w_data;
		  unsigned char data0, data1;

		  /* Data ̈̉ */
		  w_data = strtol ((char *)&(buf[3]), NULL, 16);
		  if ((w_data != LONG_MAX) && (w_data != LONG_MIN))
			{
			  data0 = (unsigned char)(w_data & 0x00ff);
			  data1 = (unsigned char)((w_data & 0xff00) >> 8);

			  /* e֐̌Ăяo */
			  ether_cmd_tbl[i].func(data0, data1, type, addr, port);
			}

		  return;
		}
	}
  /* CMD G[ */

  return;
}


/* ********************************************************** */
/* Tv@F@eCMDs ("IT")                             */
/* ֐@F@ether_cmd_initialize                             */
/* @F@unsigned char data0                              */
/*           unsigned char data1                              */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
ether_cmd_initialize (unsigned char data0, unsigned char data1, int type, unsigned char *addr, unsigned short port)
{
  /* Ԃ */
  char *buf = ":ITOK\r\n";
  int ret;

  if (type == SOCK_TCPS)
	{
	  /* TCP */
	  ret = send (type, (unsigned char *)buf, strlen(buf));
	  if (ret < 0)
		{
		  sprintf (&print_buf[0], "TCP : sendto error. %d\r\n", ret);
		  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
		  close (type);
		  return;
		}
	}
  else
	{
	  /* UDP */
	  ret = sendto (type, (unsigned char *)buf, strlen(buf), addr, port);
	  if (ret < 0)
		{
		  sprintf (&print_buf[0], "UDP : sendto error. %d\r\n", ret);
		  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
		  return;
		}
	}

  /* [hׂĂƂ߂ */
  ServoDemoApl_Set_AutoMode (0);
  SteppingDemoApl_Set_AutoMode (0);
  LedDemoApl_Set_AutoMode (0);
  AdcDemoApl_Set_AutoMode (0);

  /* ServoZ^[ɕς */
  ServoDemoApl_Set_CmdAngle (SERVO_DEMO_ANGLE_CENTER);

  /* LED S */
  LedDemoApl_Set_FreeModeData (0);
  return;
}

/* ********************************************************** */
/* Tv@F@eCMDs ("RW")                             */
/* ֐@F@ether_cmd_rtc_write                              */
/* @F@unsigned char data0                              */
/*           unsigned char data1                              */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
ether_cmd_rtc_write (unsigned char data0, unsigned char data1, int type, unsigned char *addr, unsigned short port)
{
  RTC_APL_INFO info;
  RTC_TYPE rtc_type;
  /* Ԃ */
  char *buf = ":RWOK\r\n";
  int ret;

  if (type == SOCK_TCPS)
	{
	  /* TCP */
	  ret = send (type, (unsigned char *)buf, strlen(buf));
	  if (ret < 0)
		{
		  sprintf (&print_buf[0], "TCP : sendto error. %d\r\n", ret);
		  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
		  close (type);
		  return;
		}
	}
  else
	{
	  /* UDP */
	  ret = sendto (type, (unsigned char *)buf, strlen(buf), addr, port);
	  if (ret < 0)
		{
		  sprintf (&print_buf[0], "UDP : sendto error. %d\r\n", ret);
		  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
		  return;
		}
	}

  memset (&info, 0, sizeof(info));

  switch (data1)
	{
	case RTC_CAT_SEC:
	  info.sec = data0;
	  rtc_type = RTC_TYPE_SEC;
	  break;
	case RTC_CAT_MIN:
	  info.min = data0;	
	  rtc_type = RTC_TYPE_MIN;
	  break;
	case RTC_CAT_HOUR:
	  info.hour = data0;
	  rtc_type = RTC_TYPE_HOUR;
	  break;
	case RTC_CAT_DAYS:
	  info.day = data0;
	  rtc_type = RTC_TYPE_DAY;
	  break;
	case RTC_CAT_WEEKDAYS:
	  info.weekday = data0;
	  rtc_type = RTC_TYPE_WEEKDAY;
	  break;
	case RTC_CAT_MONTH:
	  info.month = data0;
	  rtc_type = RTC_TYPE_MONTH;
	  break;
	case RTC_CAT_YEAR:
	  info.year = data0;
	  rtc_type = RTC_TYPE_YEAR;
	  break;
	default:
	  return;
	}

  RtcDemoApl_Write_Info (rtc_type, &info);
  return;
}

/* ********************************************************** */
/* Tv@F@eCMDs ("RR")                             */
/* ֐@F@ether_cmd_rtc_read                               */
/* @F@unsigned char data0                              */
/*           unsigned char data1                              */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
ether_cmd_rtc_read (unsigned char data0, unsigned char data1, int type, unsigned char *addr, unsigned short port)
{
  RTC_APL_INFO info;
  RTC_TYPE rtc_type;
  unsigned char *out_data;
  char buf[10];
  int ret = 0;

  memset (&buf[0], 0, sizeof(buf));
  memset (&info, 0, sizeof(info));

  /* f[^̐擪 */
  sprintf (&buf[0], ":RR");

  switch (data1)
	{
	case RTC_CAT_SEC:
	  out_data = &info.sec;
	  rtc_type = RTC_TYPE_SEC;
	  break;
	case RTC_CAT_MIN:
	  out_data = &info.min;
	  rtc_type = RTC_TYPE_MIN;
	  break;
	case RTC_CAT_HOUR:
	  out_data = &info.hour;
	  rtc_type = RTC_TYPE_HOUR;
	  break;
	case RTC_CAT_DAYS:
	  out_data = &info.day;
	  rtc_type = RTC_TYPE_DAY;
	  break;
	case RTC_CAT_WEEKDAYS:
	  out_data = &info.weekday;
	  rtc_type = RTC_TYPE_WEEKDAY;
	  break;
	case RTC_CAT_MONTH:
	  out_data = &info.month;
	  rtc_type = RTC_TYPE_MONTH;
	  break;
	case RTC_CAT_YEAR:
	  out_data = &info.year;
	  rtc_type = RTC_TYPE_YEAR;
	  break;
	default:
	  sprintf (&buf[3], "NG\r\n");
	  ret = -1;
	  break;
	}

  if (ret == 0)
	{
	  RtcDemoApl_Read_Info (rtc_type, &info); /* pgr0000 */
	  
	  /* 擾f[^o */
	  sprintf (&buf[3], "%02x\r\n", *out_data); /* pgr0000 */
	}

  /* { */
  if (type == SOCK_TCPS)
	{
	  /* TCP */
	  ret = send (type, (unsigned char *)buf, strlen(buf));
	  if (ret < 0)
		{
		  sprintf (&print_buf[0], "TCP : sendto error. %d\r\n", ret);
		  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
		  close (type);
		  return;
		}
	}
  else
	{
	  /* UDP */
	  ret = sendto (type, (unsigned char *)buf, strlen(buf), addr, port);
	  if (ret < 0)
		{
		  sprintf (&print_buf[0], "UDP : sendto error. %d\r\n", ret);
		  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
		  return;
		}
	}


  return;
}

/* ********************************************************** */
/* Tv@F@eCMDs ("BC")                             */
/* ֐@F@ether_cmd_lcd_light                              */
/* @F@unsigned char data0                              */
/*           unsigned char data1                              */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
ether_cmd_lcd_light (unsigned char data0, unsigned char data1, int type, unsigned char *addr, unsigned short port)
{
  /* Ԃ */
  char *buf = ":BCNG\r\n";
  int ret;

  if (type == SOCK_TCPS)
	{
	  /* TCP */
	  ret = send (type, (unsigned char *)buf, strlen(buf));
	  if (ret < 0)
		{
		  sprintf (&print_buf[0], "TCP : sendto error. %d\r\n", ret);
		  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
		  close (type);
		  return;
		}
	}
  else
	{
	  /* UDP */
	  ret = sendto (type, (unsigned char *)buf, strlen(buf), addr, port);
	  if (ret < 0)
		{
		  sprintf (&print_buf[0], "UDP : sendto error. %d\r\n", ret);
		  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
		  return;
		}
	}
  return;
}

/* ********************************************************** */
/* Tv@F@eCMDs ("LW")                             */
/* ֐@F@ether_cmd_led_write                              */
/* @F@unsigned char data0                              */
/*           unsigned char data1                              */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
ether_cmd_led_write (unsigned char data0, unsigned char data1, int type, unsigned char *addr, unsigned short port)
{
  /* Ԃ */
  char *buf = ":LWOK\r\n";
  int ret;

  if (type == SOCK_TCPS)
	{
	  /* TCP */
	  ret = send (type, (unsigned char *)buf, strlen(buf));
	  if (ret < 0)
		{
		  sprintf (&print_buf[0], "TCP : sendto error. %d\r\n", ret);
		  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
		  close (type);
		  return;
		}
	}
  else
	{
	  /* UDP */
	  ret = sendto (type, (unsigned char *)buf, strlen(buf), addr, port);
	  if (ret < 0)
		{
		  sprintf (&print_buf[0], "UDP : sendto error. %d\r\n", ret);
		  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
		  return;
		}
	}

  LedDemoApl_Set_FreeModeData (data0);
  return;
}

/* ********************************************************** */
/* Tv@F@eCMDs ("LR")                             */
/* ֐@F@ether_cmd_led_read                               */
/* @F@unsigned char data0                              */
/*           unsigned char data1                              */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
ether_cmd_led_read (unsigned char data0, unsigned char data1, int type, unsigned char *addr, unsigned short port)
{
  unsigned char out;
  char buf[10];
  int ret;

  memset (&buf[0], 0, sizeof(buf));

  /* f[^̐擪 */
  sprintf (&buf[0], ":LR");


  out = LedDemoApl_Get_LedInfo ();

  sprintf (&buf[3], "%02x\r\n", out);

  /* { */
  if (type == SOCK_TCPS)
	{
	  /* TCP */
	  ret = send (type, (unsigned char *)buf, strlen(buf));
	  if (ret < 0)
		{
		  sprintf (&print_buf[0], "TCP : sendto error. %d\r\n", ret);
		  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
		  close (type);
		  return;
		}
	}
  else
	{
	  /* UDP */
	  ret = sendto (type, (unsigned char *)buf, strlen(buf), addr, port);
	  if (ret < 0)
		{
		  sprintf (&print_buf[0], "UDP : sendto error. %d\r\n", ret);
		  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
		  return;
		}
	}

  return;
}

/* ********************************************************** */
/* Tv@F@eCMDs ("SV")                             */
/* ֐@F@ether_cmd_servo_ctl                              */
/* @F@unsigned char data0                              */
/*           unsigned char data1                              */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
ether_cmd_servo_ctl (unsigned char data0, unsigned char data1, int type, unsigned char *addr, unsigned short port)
{
  int ret;
  char buf[10];

  memset (&buf[0], 0, sizeof(buf));
  /* f[^̐擪 */
  sprintf (&buf[0], ":SV");


  ret = ServoDemoApl_Set_CmdAngle (data0);
  if (ret < 0)
	{
	  sprintf (&buf[3], "NG\r\n");
	}
  else
	{
	  sprintf (&buf[3], "OK\r\n");
	}

  /* { */
  if (type == SOCK_TCPS)
	{
	  /* TCP */
	  ret = send (type, (unsigned char *)buf, strlen(buf));
	  if (ret < 0)
		{
		  sprintf (&print_buf[0], "TCP : sendto error. %d\r\n", ret);
		  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
		  close (type);
		  return;
		}
	}
  else
	{
	  /* UDP */
	  ret = sendto (type, (unsigned char *)buf, strlen(buf), addr, port);
	  if (ret < 0)
		{
		  sprintf (&print_buf[0], "UDP : sendto error. %d\r\n", ret);
		  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
		  return;
		}
	}

  return;
}

/* ********************************************************** */
/* Tv@F@eCMDs ("SP")                             */
/* ֐@F@ether_cmd_step_ctl                               */
/* @F@unsigned char data0                              */
/*           unsigned char data1                              */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
ether_cmd_step_ctl (unsigned char data0, unsigned char data1, int type, unsigned char *addr, unsigned short port)
{
  int ret;
  char buf[10];

  memset (&buf[0], 0, sizeof(buf));
  /* f[^̐擪 */
  sprintf (&buf[0], ":SP");


  ret = SteppingDemoApl_Start_CmdMode (data0);
  if (ret < 0)
	{
	  sprintf (&buf[3], "NG\r\n");
	}
  else
	{
	  sprintf (&buf[3], "OK\r\n");
	}

  /* { */
  if (type == SOCK_TCPS)
	{
	  /* TCP */
	  ret = send (type, (unsigned char *)buf, strlen(buf));
	  if (ret < 0)
		{
		  sprintf (&print_buf[0], "TCP : sendto error. %d\r\n", ret);
		  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
		  close (type);
		  return;
		}
	}
  else
	{
	  /* UDP */
	  ret = sendto (type, (unsigned char *)buf, strlen(buf), addr, port);
	  if (ret < 0)
		{
		  sprintf (&print_buf[0], "UDP : sendto error. %d\r\n", ret);
		  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
		  return;
		}
	}

  return;
}

/* ********************************************************** */
/* Tv@F@eCMDs ("AU")                             */
/* ֐@F@ether_cmd_auto_mode                              */
/* @F@unsigned char data0                              */
/*           unsigned char data1                              */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
ether_cmd_auto_mode (unsigned char data0, unsigned char data1, int type, unsigned char *addr, unsigned short port)
{
  /* Ԃ */
  char *buf = ":AUOK\r\n";
  int ret;

  if (type == SOCK_TCPS)
	{
	  /* TCP */
	  ret = send (type, (unsigned char *)buf, strlen(buf));
	  if (ret < 0)
		{
		  sprintf (&print_buf[0], "TCP : sendto error. %d\r\n", ret);
		  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
		  close (type);
		  return;
		}
	}
  else
	{
	  /* UDP */
	  ret = sendto (type, (unsigned char *)buf, strlen(buf), addr, port);
	  if (ret < 0)
		{
		  sprintf (&print_buf[0], "UDP : sendto error. %d\r\n", ret);
		  uart_xfer ("UART0", &print_buf[0], WR, strlen(print_buf));
		  return;
		}
	}

  /* ꂼ̃[hwݒ肷 */
  /* ServoAutoMode    : Mode 1/2 */
  ServoDemoApl_Set_AutoMode (data0 & 0x03);

  /* SteppingAutoMode : Mode 3/4 */
  SteppingDemoApl_Set_AutoMode ((data0 & 0x0C) >> 2);

  /* LedAutoMode      : Mode 5/6 */
  LedDemoApl_Set_AutoMode ((data0 & 0x30) >> 4);

  /* AdcAutoMode      : Mode 7 */
  AdcDemoApl_Set_AutoMode ((data0 & 0x40) >> 6);
  return;
}
