/* ===================================================================
   \[Xt@C
   =================================================================== */

/* ********************************************************** */
/* Include */
/* ********************************************************** */
#include "ServoDemoApl.h"

/* ********************************************************** */
/* Structure & Define */
/* ********************************************************** */

/* [h */
typedef enum {
  SERVO_AUTO_NONE,
  SERVO_AUTO_LOW,
  SERVO_AUTO_MIDDLE,
  SERVO_AUTO_HIGH
} SERVO_AUTO_MODE;

/* Demo[h */
typedef enum {
  SERVO_DEMO_STATE_NONE,  /*  */
  SERVO_DEMO_STATE_AUTO,  /* [h */
  SERVO_DEMO_STATE_KEY,   /* SWD惂[h */
  SERVO_DEMO_STATE_TOUCH  /* ^b`plD惂[h */
} SERVO_DEMO_STATE;

/* ] */
typedef enum {
  SERVO_DEMO_AUTO_RIGHT,
  SERVO_DEMO_AUTO_LEFT
} SERVO_DEMO_ROT;

/* bݒl */
#define SERVO_AUTO_LOW_TIME    (75) /* ݒύX(75ms) */
#define SERVO_AUTO_MIDDLE_TIME (50)  /* ݒύX(50ms) */
#define SERVO_AUTO_HIGH_TIME   (25)  /* ݒύX(25ms) */
#define SERVO_AUTO_INTVAL_TIME_MS  (10) /* Interval Timer msl */

/* ********************************************************** */
/* Local Variable */
/* ********************************************************** */
static SERVO_AUTO_MODE servo_auto_mode;
static SERVO_DEMO_STATE servo_demo_state;
static SERVO_TIMER_STATE servo_demo_timer_state;
static int servo_demo_timer_cnt;
static SERVO_DEMO_ROT servo_demo_rot_state;

/* ********************************************************** */
/* Local ProtoType Define */
/* ********************************************************** */
static void servo_demo_set_auto_duty (void);
static void servo_demo_set_timer (void);
static int servo_demo_apl_is_sw_on (void);
/* ================================================================== */
/* ********************************************************** */
/* Src (Internal API Function) */
/* ********************************************************** */
/* ********************************************************** */
/* Tv@F@Servo DemoAv̏                         */
/* ֐@F@ServoDemoApl_Init                                */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
ServoDemoApl_Init (void)
{
  unsigned char addr;
  unsigned char data;

  servo_auto_mode        = SERVO_AUTO_NONE;
  servo_demo_state       = SERVO_DEMO_STATE_NONE;
  servo_demo_rot_state   = SERVO_DEMO_AUTO_RIGHT;
  servo_demo_timer_state = SERVO_TIMER_STOP;
  servo_demo_timer_cnt   = 0;

  /* Dipsw ͐ݒɂ */
  addr = 0x01;
  data = 0xff;
  i2c_xfer ("MCP23017", &addr, &data, WR, sizeof(data));

  ServoApl_Init ();

  return;
}


/* ********************************************************** */
/* Tv@F@Servo DemoAv̎s                           */
/* ֐@F@ServoDemoApl_Exec                                */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
ServoDemoApl_Exec (void)
{
  if (servo_auto_mode != SERVO_AUTO_NONE)
	{
	  /* Auto Mode̎w肠 */

	  if (servo_demo_state == SERVO_DEMO_STATE_AUTO)
		{
		  /* ^C}[I҂ */
		  if (servo_demo_timer_state == SERVO_TIMER_STOP)
			{
			  /* pxڍs^C}[ݒ */
			  servo_demo_set_timer ();
			  servo_demo_timer_state = SERVO_TIMER_DOING;
			  
			  /* ]ADutylvZčĐݒ */
			  servo_demo_set_auto_duty ();

			}
		}
	  else
		{
		  /* PWMU~ */
		  pwm_stop (PWM_1_NAME);
		  /* SWmF^C}[̒~ */
		  ServoApl_Stop_Timer ();

		  /* pxڍs^C}[ݒ */
		  servo_demo_set_timer ();
		  servo_demo_timer_state = SERVO_TIMER_DOING;

		  /* Duty ݒ */
		  ServoApl_Set_Duty (SERVO_APL_NOW_DUTY);
		  /* PWM Jn */
		  pwm_start (PWM_1_NAME);

		  servo_demo_state = SERVO_DEMO_STATE_AUTO;

		}

	}
  else
	{
	  if (servo_demo_state == SERVO_DEMO_STATE_AUTO)
		{
		  /* PWMU~ */
		  pwm_stop (PWM_1_NAME);
		  servo_demo_state = SERVO_DEMO_STATE_NONE;
		}

	  /* DipSw mF */
	  if (servo_demo_apl_is_sw_on () == true)
		{
		  /* SWD惂[h */
		  if (servo_demo_state == SERVO_DEMO_STATE_TOUCH)
			{
			  pwm_stop (PWM_1_NAME);
			}

		  servo_demo_state = SERVO_DEMO_STATE_KEY;
		  ServoApl_Exec ();
		}
	  else
		{
		  /* ^b`plD惂[h */
		  if (servo_demo_state == SERVO_DEMO_STATE_KEY)
			{
			  pwm_stop (PWM_1_NAME);
			  /* SWmF^C}[̒~ */
			  ServoApl_Stop_Timer ();
			}

		  /* fAvł͂̃[hȂ̂ߏȗ */

		  servo_demo_state = SERVO_DEMO_STATE_TOUCH;
		}
	}
  return;
}

/* ********************************************************** */
/* Tv@F@ServoAv Timer荞                        */
/* ֐@F@ServoApl_Irq_Timer                               */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
ServoDemoApl_Irq_Timer (void)
{
  if (servo_demo_timer_state == SERVO_TIMER_DOING)
	{
	  if (servo_demo_timer_cnt <= 0)
		{
		  servo_demo_timer_state = SERVO_TIMER_STOP;
		}
	  else
		{
		  servo_demo_timer_cnt--;
		}
	}

  ServoApl_Irq_Timer ();

  return;
}

/* ********************************************************** */
/* Tv@F@Servo AutoMode̐ݒ                             */
/* ֐@F@ServoDemoApl_Set_AutoMode                        */
/* @F@int mode                                         */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
ServoDemoApl_Set_AutoMode (int mode)
{
  servo_auto_mode = mode;
  return;
}

/* ********************************************************** */
/* Tv@F@Servo CMD̊pxw菈                          */
/* ֐@F@ServoDemoApl_Set_CmdAngle                        */
/* @F@unsigned char angle                              */
/* ߂lF@int  FI@FG[                    */
/* ********************************************************** */
int 
ServoDemoApl_Set_CmdAngle (unsigned char angle)
{
  int duty;

  /* [h쒆CMD󂯕tȂ */
  if (servo_demo_state == SERVO_DEMO_STATE_AUTO)
	{
	  return -1;
	}

  /* SWD惂[hSW͒CMD󂯕tȂ */
  if (servo_demo_state == SERVO_DEMO_STATE_KEY)
	{
	  if (ServoApl_Is_InKey () == true)
		{
		  return -1;
		}
	}

  switch (angle)
	{
	case SERVO_DEMO_ANGLE_MIN:
	  duty = SERVO_DUTY_MIN;
	  break;
	case SERVO_DEMO_ANGLE_CENTER:
	  duty = SERVO_DUTY_CENTER;
	  break;
	case SERVO_DEMO_ANGLE_MAX:
	  duty = SERVO_DUTY_MAX;
	  break;
	default:
	  duty = ((angle * (SERVO_DUTY_MAX - SERVO_DUTY_MIN - 1))/ 180) + 1;
	  break;
	}
  if (duty < SERVO_DUTY_MIN)
	{
	  duty = SERVO_DUTY_MIN;
	}

  ServoApl_Set_Duty (duty);
  pwm_start (PWM_1_NAME);
  return 0;
}

/* ================================================================== */
/* ********************************************************** */
/* Src (Local Function) */
/* ********************************************************** */

/* ********************************************************** */
/* Tv@F@Servo AutoModeDutyݒ                         */
/* ֐@F@servo_demo_set_auto_duty                         */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
servo_demo_set_auto_duty (void)
{
  int duty;

  /* ݂Dutyl擾 */
  duty = ServoApl_Get_Duty ();

  /* ]vZ */
  if (servo_demo_rot_state == SERVO_DEMO_AUTO_RIGHT)
	{
	  duty++;

	  if (duty >= SERVO_DUTY_MAX)
		{
		  duty = SERVO_DUTY_MAX;
		  /* ]ς */
		  servo_demo_rot_state = SERVO_DEMO_AUTO_LEFT;
		}
	}
  else
	{
	  duty--;
	  if (duty <= SERVO_DUTY_MIN)
		{
		  duty = SERVO_DUTY_MIN;
		  /* ]ς */
		  servo_demo_rot_state = SERVO_DEMO_AUTO_RIGHT;
		}
	}

  /* Duty ݒ */
  ServoApl_Set_Duty (duty);

  return;
}


/* ********************************************************** */
/* Tv@F@Servo Autop^C}[ݒ                         */
/* ֐@F@servo_demo_set_timer                             */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
servo_demo_set_timer (void)
{
  /* ModeԂŃ^C}[lς */
  switch (servo_auto_mode)
	{  
	case SERVO_AUTO_LOW:
	  servo_demo_timer_cnt = SERVO_AUTO_LOW_TIME;
	  break;
	case SERVO_AUTO_HIGH:
	  servo_demo_timer_cnt = SERVO_AUTO_HIGH_TIME;
	  break;
	case SERVO_AUTO_MIDDLE:
	default:
	  servo_demo_timer_cnt = SERVO_AUTO_MIDDLE_TIME;
	  break;
	}

  servo_demo_timer_cnt = servo_demo_timer_cnt * SERVO_AUTO_INTVAL_TIME_MS;

  return;
}

/* ********************************************************** */
/* Tv@F@Servo DipSw 0 ONmF                           */
/* ֐@F@servo_demo_apl_is_sw_on                          */
/* @F@Ȃ                                             */
/* ߂lF@int true ON/ false OFF                           */
/* ********************************************************** */
static int 
servo_demo_apl_is_sw_on (void)
{
  unsigned char dp_sw;
  unsigned char addr = 0x13;

  /* DipSw̏擾(addr ͎bH) */
  i2c_xfer ("MCP23017", &addr, &dp_sw, RE, sizeof(dp_sw));

  /* SW 0 ̏mF */

  if (dp_sw & 0x01)
	{
	  return false;
	}

  return true;
}
