/* ===================================================================
   \[Xt@C
   =================================================================== */

/* ********************************************************** */
/* Include */
/* ********************************************************** */
#include "ServoSampleApl.h"

/* ********************************************************** */
/* Structure & Define */
/* ********************************************************** */
/* KEY ͏ */
typedef enum {
  SERVO_KEY_IN_NONE,
  SERVO_KEY_IN_A,
  SERVO_KEY_IN_B,
  SERVO_KEY_IN_C
} SERVO_KEY_IN_TYPE;

/* bݒl */
#define SERVO_TIME            (50) /* ݒύX(50ms) */
#define SERVO_INTVAL_TIME_MS  (10) /* Interval Timer msl */


/* ********************************************************** */
/* Local Variable */
/* ********************************************************** */
static SERVO_KEY_IN_TYPE servo_key_in;
static int servo_apl_duty;
static SERVO_TIMER_STATE servo_timer_state;
static int servo_timer_cnt;

/* ********************************************************** */
/* Local ProtoType Define */
/* ********************************************************** */
static void servo_apl_set_duty (SERVO_KEY_IN_TYPE key_type);
static void servo_apl_set_timer (void);
static int servo_apl_is_input (SERVO_KEY_IN_TYPE *input_key);
static int servo_apl_is_release (SERVO_KEY_IN_TYPE input_key);

/* ================================================================== */
/* ********************************************************** */
/* Src (Internal API Function) */
/* ********************************************************** */
/* ********************************************************** */
/* Tv@F@ServoAv̏                              */
/* ֐@F@ServoApl_Init                                    */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
ServoApl_Init (void)
{
  servo_key_in      = SERVO_KEY_IN_NONE;
  servo_timer_state = SERVO_TIMER_STOP;
  servo_timer_cnt   = 0;
  servo_apl_duty    = SERVO_DUTY_CENTER;

  return;
}

/* ********************************************************** */
/* Tv@F@ServoAv̎s                                */
/* ֐@F@ServoApl_Exec                                    */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
ServoApl_Exec (void)
{

  if (servo_key_in != SERVO_KEY_IN_NONE)
	{
	  /* SWꂽǂmF */
	  if (servo_apl_is_release (servo_key_in) == true)
		{
		  /* SWꂽ̂PWMƂ߂ */
		  pwm_stop (PWM_1_NAME);
		  
		  servo_timer_state = SERVO_TIMER_STOP;
		  servo_timer_cnt   = 0;
		  servo_key_in = SERVO_KEY_IN_NONE;
		}
	  else
		{
		  /* SWmF^C}[I҂ */
		  if (servo_timer_state == SERVO_TIMER_STOP)
			{
			  /* ^C}[X^[g */
			  servo_apl_set_timer ();
		  
			  /* ^C}[쒆ݒ */
			  servo_timer_state = SERVO_TIMER_DOING;

			  /* Dutyl̐ݒ */
			  servo_apl_set_duty (servo_key_in);

			  pwm_duty (PWM_1_NAME, servo_apl_duty);

			}
		}
	}
  else
	{
	  SERVO_KEY_IN_TYPE input_key;

	  /* LSW͊mF */
	  if (servo_apl_is_input (&input_key) == true)
		{
		  /* ^C}[X^[g */
		  servo_apl_set_timer ();
		  
		  /* ^C}[쒆ݒ */
		  servo_timer_state = SERVO_TIMER_DOING;
		  

		  /* ݂̐ݒlPWMJn */
		  pwm_duty (PWM_1_NAME, servo_apl_duty);
		  pwm_start (PWM_1_NAME);

		  /* Keyۑ */
		  servo_key_in = input_key;
		}
	}

  return;
}

/* ********************************************************** */
/* Tv@F@ServoAv Timer荞                        */
/* ֐@F@ServoApl_Irq_Timer                               */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
ServoApl_Irq_Timer (void)
{
  if (servo_timer_state == SERVO_TIMER_DOING)
	{
	  if (servo_timer_cnt <= 0)
		{
		  servo_timer_state = SERVO_TIMER_STOP;
		}
	  else
		{
		  servo_timer_cnt--;
		}
	}

  return;
}

/* ********************************************************** */
/* Tv@F@ServoAv Timer~                            */
/* ֐@F@ServoApl_Stop_Timer                              */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
ServoApl_Stop_Timer (void)
{
  servo_timer_state = SERVO_TIMER_STOP;
  servo_timer_cnt   = 0;

  return;
}

/* ********************************************************** */
/* Tv@F@ServoAv Dutyݒ                             */
/* ֐@F@ServoApl_Set_Duty                                */
/* @F@int duty                                         */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
ServoApl_Set_Duty (int duty)
{

  if (duty == SERVO_APL_NOW_DUTY)
	{
	  /* ݂̐ݒlPWMJn */
	  pwm_duty (PWM_1_NAME, servo_apl_duty);
	}
  else
	{
	  servo_apl_duty = duty;
	  pwm_duty (PWM_1_NAME, servo_apl_duty);
	}
  return;
}

/* ********************************************************** */
/* Tv@F@ServoAv Dutyݒl̎擾                     */
/* ֐@F@ServoApl_Get_Duty                                */
/* @F@Ȃ                                             */
/* ߂lF@int                                              */
/* ********************************************************** */
int 
ServoApl_Get_Duty (void)
{
  return servo_apl_duty;
}

/* ********************************************************** */
/* Tv@F@ServoAv Key͏Ԃ̎擾                    */
/* ֐@F@ServoApl_Is_InKey                                */
/* @F@Ȃ                                             */
/* ߂lF@int                                              */
/* ********************************************************** */
int 
ServoApl_Is_InKey (void)
{
  if (servo_key_in != SERVO_KEY_IN_NONE)
	{
	  return true;
	}
  return false;
}


/* ================================================================== */
/* ********************************************************** */
/* Src (Local Function) */
/* ********************************************************** */
/* ********************************************************** */
/* Tv@F@ServoAv Duty̐ݒ                           */
/* ֐@F@servo_apl_set_duty                               */
/* @F@SERVO_KEY_IN_TYPE key_type : ͒SW          */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
servo_apl_set_duty (SERVO_KEY_IN_TYPE key_type)
{
  switch (key_type)
	{
	case SERVO_KEY_IN_A:
	  if (servo_apl_duty < SERVO_DUTY_MAX)
		{
		  servo_apl_duty++;
		}
	  break;
	case SERVO_KEY_IN_B:
	  if (servo_apl_duty > SERVO_DUTY_MIN)
		{
		  servo_apl_duty--;
		}
	  break;
	case SERVO_KEY_IN_C:
	  servo_apl_duty = SERVO_DUTY_CENTER;
	  break;
	default:
	  break;
	}

  return;
}

/* ********************************************************** */
/* Tv@F@T[{Aṽ^C}[lݒ                     */
/* ֐@F@servo_apl_set_timer                              */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
servo_apl_set_timer (void)
{
  servo_timer_cnt = SERVO_TIME * SERVO_INTVAL_TIME_MS;  /* 50ms : l͎b */
  return;
}

/* ********************************************************** */
/* Tv@F@LInput邩ǂ                          */
/* ֐@F@servo_apl_is_input                               */
/* @F@SERVO_KEY_IN_TYPE *input_key: LSWi[       */
/* ߂lF@true :  / false F Ȃ                      */
/* ********************************************************** */
static int 
servo_apl_is_input (SERVO_KEY_IN_TYPE *input_key)
{
  if (pio_input (PD4) == 1)
	{
	  *input_key = SERVO_KEY_IN_A;
	  return true;
	}

  if (pio_input (PD5) == 1)
	{
	  *input_key = SERVO_KEY_IN_B;
	  return true;
	}

  if (pio_input (PD6) == 1)
	{
	  *input_key = SERVO_KEY_IN_C;
	  return true;
	}

  *input_key = SERVO_KEY_IN_NONE;
  return false;
}

/* ********************************************************** */
/* Tv@F@SWꂽǂ                             */
/* ֐@F@servo_apl_is_release                             */
/* @F@SERVO_KEY_IN_TYPE input_key: ׂSW            */
/* ߂lF@true : ꂽ / false F Ȃ                  */
/* ********************************************************** */
static int 
servo_apl_is_release (SERVO_KEY_IN_TYPE input_key)
{
  unsigned char chk_key;

  switch (input_key)
	{
	case SERVO_KEY_IN_A:
	  chk_key = PD4;
	  break;
	case SERVO_KEY_IN_B:
	  chk_key = PD5;
	  break;
	case SERVO_KEY_IN_C:
	  chk_key = PD6;
	  break;
	default:
	  return true;
	}

  /* YSWꂽǂ`FbN */
  if (pio_input (chk_key) != 1)
	{
	  return true;
	}

  return false;
}


