/* ===================================================================
   \[Xt@C
   =================================================================== */

/* ********************************************************** */
/* Include */
/* ********************************************************** */
#include "SteppingDemoApl.h"


/* ********************************************************** */
/* Structure & Define */
/* ********************************************************** */
/* [h */
typedef enum {
  STEPPING_AUTO_NONE,
  STEPPING_AUTO_RIGHT,
  STEPPING_AUTO_LEFT,
  STEPPING_AUTO_TURN
} STEPPING_AUTO_MODE;

/* Demo [h */
typedef enum {
  STEPPING_DEMO_STATE_NONE, /*  */
  STEPPING_DEMO_STATE_AUTO, /* [h */
  STEPPING_DEMO_STATE_KEY,  /* SWD惂[h */
  STEPPING_DEMO_STATE_CMD	/* CMD[h */
} STEPPING_DEMO_STATE;

/* ] */
typedef enum {
  STEPPING_DEMO_AUTO_RIGHT, 
  STEPPING_DEMO_AUTO_LEFT
} STEPPING_DEMO_ROT;

/* bݒl */
#define STEPPING_DEMO_AUTO_TIME    (5) /* ݒύX(5ms) */
#define STEPPING_DEMO_AUTO_INTVAL_TIME_MS  (10) /* Interval Timer msl */
#define STEPPING_ROT_CNT  (55)

/* ********************************************************** */
/* Local Variable */
/* ********************************************************** */
static STEPPING_AUTO_MODE stepping_auto_mode;
static STEPPING_DEMO_STATE stepping_demo_state;
static STEPPING_TIMER_STATE stepping_demo_timer_state;
static int stepping_demo_timer_cnt;
static STEPPING_DEMO_ROT stepping_demo_rot_state;
static int stepping_rot_cnt;
static int stepping_pulse_cnt;  /* CMD[h̏o̓pX */

/* ********************************************************** */
/* Local ProtoType Define */
/* ********************************************************** */
static void stepping_demo_auto_start (void);
static void stepping_demo_set_timer (void);
static int stepping_demo_apl_is_sw_off (void);
static void stepping_demo_cmd_mode (void);

/* ================================================================== */
/* ********************************************************** */
/* Src (Internal API Function) */
/* ********************************************************** */

/* ********************************************************** */
/* Tv@F@Stepping DemoAv̏                      */
/* ֐@F@SteppingDemoApl_Init                             */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
SteppingDemoApl_Init (void)
{
  stepping_auto_mode        = STEPPING_AUTO_NONE;
  stepping_demo_state       = STEPPING_DEMO_STATE_NONE;
  stepping_demo_rot_state   = STEPPING_DEMO_AUTO_RIGHT;
  stepping_demo_timer_state = STEPPING_TIMER_STOP;
  stepping_demo_timer_cnt   = 0;
  stepping_rot_cnt          = STEPPING_ROT_CNT;

  SteppingApl_Init ();

  return;
}

/* ********************************************************** */
/* Tv@F@Stepping DemoAv̎s                        */
/* ֐@F@SteppingDemoApl_Exec                             */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
SteppingDemoApl_Exec (void)
{
  if (stepping_auto_mode != STEPPING_AUTO_NONE)
	{
	  /* Auto Mode̎w肠 */

	  if (stepping_demo_state == STEPPING_DEMO_STATE_AUTO)
		{
		  /* ^C}[I҂ */
		  if (stepping_demo_timer_state == STEPPING_TIMER_STOP)
			{
			  stepping_demo_auto_start ();
			}
		}
	  else
		{
		  /* [hɈڍs */
		  stepping_demo_state = STEPPING_DEMO_STATE_AUTO;
		  stepping_rot_cnt = STEPPING_ROT_CNT;
		  stepping_demo_auto_start ();
		}
	}
  else
	{
	  if (stepping_demo_state == STEPPING_DEMO_STATE_AUTO)
		{
		  /* ~ */
		  SteppingApl_Stop ();
		  stepping_demo_timer_state = STEPPING_TIMER_STOP;
		  stepping_demo_state = STEPPING_DEMO_STATE_NONE;

		}

	  if (stepping_demo_state == STEPPING_DEMO_STATE_CMD)
		{
		  /* ^C}[I҂ */
		  if (stepping_demo_timer_state == STEPPING_TIMER_STOP)
			{
			  stepping_demo_cmd_mode ();
			}
		}
	  else
		{
		  /* DipSW mF */
		  if (stepping_demo_apl_is_sw_off () == true)
			{
			  /* SWD惂[h */
			  stepping_demo_state = STEPPING_DEMO_STATE_KEY;
			  SteppingApl_Exec ();
			}
		  else
			{
			  if (stepping_demo_state == STEPPING_DEMO_STATE_KEY)
				{
				  /* ~ */
				  SteppingApl_Stop ();

				  stepping_demo_state = STEPPING_DEMO_STATE_NONE;
				}
			}
		}
	}

  return;
}

/* ********************************************************** */
/* Tv@F@SteppingAv Timer荞                     */
/* ֐@F@SteppingApl_Irq_Timer                            */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
SteppingDemoApl_Irq_Timer (void)
{
  if (stepping_demo_timer_state == STEPPING_TIMER_DOING)
	{
	  if (stepping_demo_timer_cnt <= 0)
		{
		  stepping_demo_timer_state = STEPPING_TIMER_STOP;
		}
	  else
		{
		  stepping_demo_timer_cnt--;
		}
	}

  SteppingApl_Irq_Timer ();

  return;
}

/* ********************************************************** */
/* Tv@F@Stepping AutoMode̐ݒ                          */
/* ֐@F@SteppingDemoApl_Set_AutoMode                     */
/* @F@int mode                                         */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
SteppingDemoApl_Set_AutoMode (int mode)
{
  stepping_auto_mode = mode;
  return;
}

/* ********************************************************** */
/* Tv@F@Stepping CmdMode̊Jn                           */
/* ֐@F@SteppingDemoApl_Start_CmdMode                    */
/* @F@unsigned char pulse_cnt                          */
/* ߂lF@int FI@FG[                     */
/* ********************************************************** */
int 
SteppingDemoApl_Start_CmdMode (unsigned char pulse_cnt)
{
  /* [h쒆CMD󂯕tȂ */
  /* CMD[h󂯕tȂ */
  if ((stepping_demo_state == STEPPING_DEMO_STATE_AUTO) ||
	  (stepping_demo_state == STEPPING_DEMO_STATE_CMD))
	{
	  return -1;
	}

  /* SWD惂[hSW͒CMD󂯕tȂ */
  if (stepping_demo_state == STEPPING_DEMO_STATE_KEY)
	{
	  if (SteppingApl_Is_InKey () == true)
		{
		  return -1;
		}
	}

  stepping_demo_state = STEPPING_DEMO_STATE_CMD;
  stepping_pulse_cnt  = pulse_cnt;
  /* Timerݒ */
  stepping_demo_set_timer ();
  stepping_demo_timer_state = STEPPING_TIMER_DOING;

  return 0;
}

/* ================================================================== */
/* ********************************************************** */
/* Src (Local Function) */
/* ********************************************************** */
/* ********************************************************** */
/* Tv@F@Stepping̎[h@@                         */
/* ֐@F@stepping_demo_auto_start                         */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
stepping_demo_auto_start (void)
{
  STEPPING_STATE state;

  /* Timerݒ */
  stepping_demo_set_timer ();

  stepping_demo_timer_state = STEPPING_TIMER_DOING;

  /* ݂̏ԂŊJn */
  SteppingApl_Start ();

  /* ̏Ԃ֕ύX */
  state = SteppingApl_Get_State ();

  switch (stepping_auto_mode)
	{
	case STEPPING_AUTO_RIGHT:
	  if (state == STEPPING_STATE_DA)
		{
		  state = STEPPING_STATE_A;
		}
	  else
		{
		  state++;
		}
	  break;
	case STEPPING_AUTO_LEFT:
	  if (state == STEPPING_STATE_A)
		{
		  state = STEPPING_STATE_DA;
		}
	  else
		{
		  state--;
		}
	  break;
	case STEPPING_AUTO_TURN:
	  if (stepping_demo_rot_state == STEPPING_DEMO_AUTO_RIGHT)
		{
		  if (state == STEPPING_STATE_DA)
			{
			  if (stepping_rot_cnt == 0)
				{
				  stepping_demo_rot_state = STEPPING_DEMO_AUTO_LEFT;
				  stepping_rot_cnt = STEPPING_ROT_CNT;
				}
			  else
				{
				  state = STEPPING_STATE_A;
				  stepping_rot_cnt--;
				}

			}
		  else
			{
			  state++;
			}
		}
	  else
		{
		  if (state == STEPPING_STATE_A)
			{
			  if (stepping_rot_cnt == 0)
				{
				  stepping_demo_rot_state = STEPPING_DEMO_AUTO_RIGHT;
				  stepping_rot_cnt = STEPPING_ROT_CNT;
				}
			  else
				{
				  state = STEPPING_STATE_DA;
				  stepping_rot_cnt--;
				}
			}
		  else
			{
			  state--;
			}
		}
	  break;
	default:
	  break;
	}

  SteppingApl_Set_State (state);

  return;
}

/* ********************************************************** */
/* Tv@F@Stepping̃^C}[lݒ                         */
/* ֐@F@stepping_demo_set_timer                          */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
stepping_demo_set_timer (void)
{
  stepping_demo_timer_cnt = STEPPING_DEMO_AUTO_TIME * STEPPING_DEMO_AUTO_INTVAL_TIME_MS;  /* 5ms : l͎b */
  return;
}

/* ********************************************************** */
/* Tv@F@Stepping DipSw 0 OFFmF                       */
/* ֐@F@stepping_demo_apl_is_sw_off                      */
/* @F@Ȃ                                             */
/* ߂lF@int true ON/ false OFF                           */
/* ********************************************************** */
static int 
stepping_demo_apl_is_sw_off (void)
{
  unsigned char dp_sw;
  unsigned char addr = 0x13;

  /* DipSw̏擾(addr ͎bH) */
  i2c_xfer ("MCP23017", &addr, &dp_sw, RE, sizeof(dp_sw));

  /* SW 0 ̏mF */

  if (dp_sw & 0x01)
	{
	  return true;
	}

  return false;
}

/* ********************************************************** */
/* Tv@F@SteppingCMD[h@ @                         */
/* ֐@F@stepping_demo_cmd_mode                           */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
stepping_demo_cmd_mode (void)
{
  STEPPING_STATE state;

  /* Timerݒ */
  stepping_demo_set_timer ();
  stepping_demo_timer_state = STEPPING_TIMER_DOING;


  /* ݂̏ԂŊJn */
  SteppingApl_Start ();

  /* ̏Ԃ֕ύX */
  state = SteppingApl_Get_State ();
  if (state == STEPPING_STATE_DA)
	{
	  state = STEPPING_STATE_A;
	}
  else
	{
	  state++;
	}
  SteppingApl_Set_State (state);

  /* o̓pX̍XV */
  if (stepping_pulse_cnt != 0)
	{
	  stepping_pulse_cnt--;
	}

  if (stepping_pulse_cnt == 0)
	{
	  stepping_demo_state = STEPPING_DEMO_STATE_NONE;
	  stepping_demo_timer_state = STEPPING_TIMER_STOP;
	}

  return;
}
